from asita.utils.http_types import HttpMethods

class Route():

    def __init__(self, path: str, request_type: HttpMethods, callback):
        self.path = path
        self.request_type = request_type
        self.callback = callback
    
    def get_method(self) -> str:
        return self.request_type

    def get_path(self) -> str:
        return self.path

    def get_callback(self):
        return self.callback

    def handle(self, request: object, response: object):
        self.callback(request, response)

    def has_method(self, method) -> bool:
        if not isinstance(method, HttpMethods):
            raise Exception("Please make sur your method is an instance of HttpMethods.")
        return method == self.get_method()


from .route import Route
from ..utils.http_types import HttpMethods

class Router:

    def __init__(self, default_path: str = ""):
        self.default_path = default_path
        self.routes = list()

    def use(self, path: str, router: object):
        if isinstance(router, Router):
            raise Exception("router must be Router object.")
        router: Router = router(path)
        for route in router.get_routes():
            self._add_route(route.get_path(), route.get_method(), route.get_callback())

    def all(self, path: str, callback):
        self._add_route(path, HttpMethods.ALL, callback)

    def post(self, path: str, callback):
        self._add_route(path, HttpMethods.POST, callback)

    def get(self, path: str, callback):
        self._add_route(path, HttpMethods.GET, callback)

    def put(self, path: str, callback):
        self._add_route(path, HttpMethods.PUT, callback)

    def patch(self, path: str, callback):
        self._add_route(path, HttpMethods.PATCH, callback)
        
    def delete(self, path: str, callback):
        self._add_route(path, HttpMethods.DELETE, callback)

    def head(self, path: str, callback):
        self._add_route(path, HttpMethods.HEAD, callback)

    def _add_route(self, path: str, request_type: HttpMethods, callback):
        self.routes.append({
            "path": self.default_path + path,
            "route": Route(path, request_type, callback)
        })

    def get_routes(self) -> list:
        return self.routes


from http.server import HTTPServer

from .handlers.http_request_handler import HttpRequestHandler

from .router.router import Router

class Application(Router):

    def __init__(self):
        super().__init__()

        self.instance = None
        self.asset_directory = {}
        pass

    def define_asset(self, name: str, directory: str):
        self.asset_directory = {
            "name": name,
            "directory": directory
        }

    def listen(self, port: int, callback):
        if not port and type(port) is not int:
            callback(ValueError("No port found."))
        try:
            callback(None)
            self.instance = HTTPServer(('localhost', port), self._get_handler)
            self.instance.serve_forever()
        except:
            callback(ConnectionError("Impossible connection"))

    def _get_handler(self, *args):
        HttpRequestHandler(self.get_routes(), self.asset_directory, *args)


import uuid

from .session import Session

class Sessions:

    def __init__(self):
        self.sessions = dict()

    def add(self) -> Session:
        sessionId = uuid.uuid1().hex
        session = Session(sessionId)
        self.sessions[sessionId] = session
        return session

    def all(self) -> object:
        return self.sessions.items()

    def has(self, sessionId: str) -> bool:
        return sessionId in self.sessions

    def get(self, sessionId: str) -> Session:
        if self.has(sessionId):
            return self.sessions[sessionId]
        return Session(Sessions.random_session_id())

    def delete(self, sessionId: str):
        del self.sessions[sessionId]
        return self

    @staticmethod
    def random_session_id() -> str:
        return uuid.uuid1().hex


from setuptools import setup

setup(
    name = "asita",
    version = "0.2.5",
    author = "Mattéo Gaillard",
    description = "A web application framework",
    long_description = open("README.md", "r").read(),
    long_description_content_type = "text/markdown",
    url = "https://github.com/Matteo0810/Asia",
    packages = [
        "asita",
        "asita.handlers",
        "asita.utils",
        "asita.sessions",
        "asita.router"
    ],
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires = '>=3.9',
)


class Session:

    def __init__(self, sessionId: str):
        self.session = {}
        self.sessionId = sessionId

    def get_session_id(self):
        return self.sessionId

    def set(self, key: str, value: str):
        self.session[key] = value
    
    def get(self, key: str):
        if self.has(key):
            return self.session[key]
        return None

    def has(self, key: str):
        return key in self.session

    def all(self) -> object:
        return self.session.items()

    def delete(self, key: str):
        del self.session[key]

    def __str__(self) -> str:
        return "sessionId=" + self.sessionId + ";Path=\"/\";Max-Age=2592000"